'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babylon = require('babylon');

var _generator = require('@babel/generator');

var _generator2 = _interopRequireDefault(_generator);

var _lodash = require('lodash.get');

var _lodash2 = _interopRequireDefault(_lodash);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var readFile = function readFile(filename) {
  return _fs2.default.readFileSync(filename, { encoding: 'UTF-8' });
};

var createFolder = function createFolder(pathname) {
  return !_fs2.default.existsSync(pathname) && _fs2.default.mkdirSync(pathname);
};

var writeFile = function writeFile(filename, data) {
  if (_fs2.default.existsSync(filename)) {
    _fs2.default.unlinkSync(filename);
  }
  _fs2.default.writeFileSync(filename, data);
};

var getCurrentJSON = function getCurrentJSON(generatedFileName) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var filename = _path2.default.resolve(process.cwd(), options.path, generatedFileName);
  if (_fs2.default.existsSync(filename)) {
    // eslint-disable-next-line
    return require(filename);
  }
  return {};
};

var mergeMaps = function mergeMaps(filesMapped) {
  return filesMapped.reduce(function (previous, current) {
    return new _map2.default([].concat((0, _toConsumableArray3.default)(previous), (0, _toConsumableArray3.default)(current)));
  }, new _map2.default());
};

var generateCodeToLanguage = function generateCodeToLanguage(translationMap, language) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var generatedFileName = 'translation.' + language + '.js';
  var json = getCurrentJSON(generatedFileName, options);
  var object = {};
  translationMap.forEach(function (value, key) {
    var item = (0, _defineProperty3.default)({}, key, {
      phrase: value,
      translation: (0, _lodash2.default)(json, key + '.translation', '')
    });
    object = (0, _extends3.default)({}, object, item);
  });
  var code = 'module.exports = ' + (0, _stringify2.default)(object);
  var ast = (0, _babylon.parse)(code, { sourceType: 'module' });
  writeFile(_path2.default.resolve(options.path, generatedFileName), (0, _generator2.default)(ast).code);
};

var generator = function () {
  var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
    var files = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var languages = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var defaultOptions, filesMapped, allTranslations;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            defaultOptions = (0, _extends3.default)({
              path: './translations'
            }, options);
            _context2.prev = 1;
            _context2.next = 4;
            return _promise2.default.all(files.map(function () {
              var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(file) {
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        console.info('generator:', file, options);
                        return _context.abrupt('return', (0, _parser2.default)(readFile(file), options));

                      case 2:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, undefined);
              }));

              return function (_x6) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 4:
            filesMapped = _context2.sent;
            allTranslations = mergeMaps(filesMapped);

            createFolder(defaultOptions.path);
            languages.forEach(function (value) {
              return generateCodeToLanguage(allTranslations, value, defaultOptions);
            });
            _context2.next = 13;
            break;

          case 10:
            _context2.prev = 10;
            _context2.t0 = _context2['catch'](1);

            console.error('Generator:', _context2.t0);

          case 13:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined, [[1, 10]]);
  }));

  return function generator() {
    return _ref.apply(this, arguments);
  };
}();

exports.default = generator;