'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getASTFromCode = exports.getTranslationsMap = exports.getModuleIdentifier = undefined;

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _babylon = require('babylon');

var _strman = require('strman');

var _traverse = require('@babel/traverse');

var _traverse2 = _interopRequireDefault(_traverse);

var _types = require('@babel/types');

var types = _interopRequireWildcard(_types);

var _lodash = require('lodash.get');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getModuleIdentifierByRequire = function getModuleIdentifierByRequire(path, moduleName) {
  var declarations = (0, _lodash2.default)(path, 'node.declarations');
  var identifier = declarations.find(function (declaration) {
    return (0, _lodash2.default)(declaration, 'init.callee.name') === 'require' && (0, _lodash2.default)(declaration, 'init.arguments[0].value') === moduleName;
  });
  return (0, _lodash2.default)(identifier, 'id.name');
};

var getModuleIdentifierByImport = function getModuleIdentifierByImport(path, moduleName) {
  if ((0, _lodash2.default)(path, 'node.source.value') !== moduleName) {
    return null;
  }
  var specifiers = (0, _lodash2.default)(path, 'node.specifiers');
  var identifier = specifiers.find(function (specifier) {
    return types.isImportDefaultSpecifier(specifier);
  });
  return (0, _lodash2.default)(identifier, 'local.name');
};

var getModuleIdentifier = exports.getModuleIdentifier = function getModuleIdentifier(ast, moduleName) {
  return new _promise2.default(function (resolve) {
    (0, _traverse2.default)(ast, {
      ImportDeclaration: function ImportDeclaration(path) {
        var identifier = getModuleIdentifierByImport(path, moduleName);
        if (identifier) {
          resolve(identifier);
        }
      },
      VariableDeclaration: function VariableDeclaration(path) {
        var identifier = getModuleIdentifierByRequire(path, moduleName);
        if (identifier) {
          resolve(identifier);
        }
      }
    });
    resolve('');
  });
};

var getTranslationValueFromIdentifier = function getTranslationValueFromIdentifier(path) {
  var init = (0, _lodash2.default)(path, 'node.init');
  if (!init) {
    return null;
  }
  if (types.isStringLiteral(init)) {
    return init.value;
  }
  if (types.isIdentifier(init)) {
    var name = init.name;

    return getTranslationValueFromIdentifier((0, _lodash2.default)(path, 'scope.bindings.' + name + '.path'));
  }
  return null;
};

var getTranslationsMap = exports.getTranslationsMap = function getTranslationsMap(ast, identifier) {
  var translations = new _map2.default();
  (0, _traverse2.default)(ast, {
    CallExpression: function CallExpression(path) {
      if ((0, _lodash2.default)(path, 'node.callee.name') !== identifier) {
        return;
      }
      var argument = (0, _lodash2.default)(path, 'node.arguments[0]');
      if (types.isStringLiteral(argument)) {
        translations.set((0, _strman.base64encode)(argument.value), argument.value);
        return;
      }
      var translation = getTranslationValueFromIdentifier((0, _lodash2.default)(path, 'scope.bindings.' + argument.name + '.path'));
      if (translation) {
        translations.set((0, _strman.base64encode)(translation), translation);
      }
    }
  });
  return translations;
};

var getASTFromCode = exports.getASTFromCode = function getASTFromCode(file) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return (0, _babylon.parse)(file, options);
};